/*
    *
    * This file is a part of CoreUniverse.
    * Shows releated information of apps from CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include "coreuniverse.h"
#include "ui_coreuniverse.h"

#include <QTextStream>
#include <QFile>
#include <QFontDatabase>

#include <cprime/cprime.h>


coreuniverse::coreuniverse(QWidget *parent): QWidget(parent), ui(new Ui::coreuniverse)
{
    ui->setupUi(this);

    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, Qt::transparent);
    setPalette(pltt);

    loadSettings();
    startSetup();

    on_about_clicked();
}

coreuniverse::~coreuniverse()
{
    delete ui;
}

/**
 * @brief Setup ui elements
 */
void coreuniverse::startSetup()
{
    QFont font;
    font.setFamily(QFontDatabase::systemFont(QFontDatabase::GeneralFont).family());
    this->setFont(font);

    QTextCursor c = ui->pgother->textCursor();
    QTextCharFormat format = c.charFormat();

    QFont fontyour = font;
    format.setFont(fontyour);
    c.setCharFormat(format);

    // set stylesheet
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());
    ui->sideView->setFixedWidth(sideViewSize);

    // sideView collapse if touchMode is enabled
    on_appTitle_clicked();

    // set window size
    if (touch) {
        this->setWindowState(Qt::WindowMaximized);
    } else {
        this->resize(800, 500);
    }

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }
}

/**
 * @brief Loads application settings
 */
void coreuniverse::loadSettings()
{
    // Sideview is visible in default
    sideView = 1;

    sideViewSize = sm->value("CoreApps", "SideViewSize");
    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    touch = sm->value("CoreApps", "TouchMode");
}

void coreuniverse::pageClick(QToolButton *btn, int i, QString title)
{
    // all button checked false
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        b->setChecked(false);
    }

    btn->setChecked(true);
    ui->selectedsection->setText(title);
    this->setWindowTitle(title + " - CoreUniverse");
    ui->pages->setCurrentIndex(i);
}

void coreuniverse::on_releaseNotes_clicked()
{
    pageClick(ui->releaseNotes, 2, tr("Release Notes"));
    QFile p(":/docs/ReleaseNotes");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    ui->pgother->setText(o.readAll());
}

void coreuniverse::on_about_clicked()
{
    pageClick(ui->about, 0, tr("About"));
}

void coreuniverse::on_credits_clicked()
{
    pageClick(ui->credits, 1, tr("Credits"));
    QString str = QString("Developers.\n"
                          "* Abrar. IRC/Matrix s96abrar:matrix.org.\n"
                          "* Marcus Britanicus. IRC/Matrix marcusbritanicus:matrix.org\n"
                          "* Saber. IRC/Matrix rahmanshaber:matrix.org\n\n"
                          "Tester.\n"
                          "* Librewish.\n"
                          "* Strit.");
    ui->pgother->setAcceptRichText(true);
    ui->pgother->setText(str);
}

void coreuniverse::on_helpUs_clicked()
{
    pageClick(ui->helpUs, 1, tr("Help Us"));
    QString str = QString("We need help desperately. We are not professional programmers. We made this apps from our own needs.\n\n"
                          "All the CoreApps are open-source with GPLv3 or later LICENSE. So do what you want with the code but remmember to share your changes.\n\n"
                          "1. Some of the apps has old code, so they have some issue in them. Please review the code and help us fix them.\n\n"
                          "2. We want to have DEB, Snap and Flatpack package of this app. If any one have experience on them please make them and send the steps and yaml or scripts in email. Video tutorial will be best.\n\n"
                          "3. We need your feedback to improve the CoreApps. Send us your feedback through email or gitlab issues.\n\n"
                          "4. You can tell us your missing app and we will try to make it with CoreApps features.");
    ui->pgother->setText(str);
}

void coreuniverse::on_recommendation_clicked()
{
    pageClick(ui->recommendation, 1, tr("Recommendation"));
    QString str = QString("Here you will find some random recommendations and tweaks and tricks to take most out of CoreApps.\n\n"
                          "* Use Qt5ct to change theme and icons and use Fusion as style.\n"
                          "* Use Kvantum to customize the themes.\n"
                          "* Please delete the coreapps.config file from home folder if you saw any ui or settings issues.\n"
                          "* Checkout other Qt apps from Manjaro's Qt app list from https://wiki.manjaro.org/index.php?title=List_of_Qt_Applications .");
    ui->pgother->setText(str);
}


void coreuniverse::on_appTitle_clicked()
{
    if (sideView && touch) {
        // all button without text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonIconOnly);
            b->setFixedWidth(sideViewIconSize.width() + 10);
        }

        ui->sideView->setFixedWidth(sideViewIconSize.width() + 10);
        sideView = 0;
    } else {
        // all button with text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
            b->setFixedWidth(sideViewSize);
        }

        ui->sideView->setFixedWidth(sideViewSize);
        sideView = 1;
    }
}


void coreuniverse::on_copyright_clicked()
{
    pageClick(ui->copyright, 1, tr("Copyright"));
    QFile p(":/docs/License");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    QString str = QString("The different components are copyrighted by their respective authors.\n\n"
                          "\n"
                          "The 20 Apps packages including this one are distributed under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 or later of the License, or (at your option) any later version.\n"
                          "\n"
                          "\n"
                         );
    ui->pgother->setText(str + o.readAll());
}



void coreuniverse::on_changelog_clicked()
{
    pageClick(ui->changelog, 1, tr("ChangeLog"));
    QFile p(":/docs/ChangeLog");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    ui->pgother->setText(o.readAll());
}

void coreuniverse::on_appSuite_clicked()
{
    pageClick(ui->appSuite, 1, tr("App Suite"));
    QFile p(":/docs/about.html");
    p.open(QIODevice::ReadOnly | QIODevice::Text);
    QTextStream o(&p);
    ui->pgother->setText(o.readAll());
}
